local addon, T = ...;
local bars = {};

--this changes the box size
local BOX_SIZE = 15;

--these two variables control where the charges are displayed in relation to the center of the screen
local X_OFFSET = 0;
local Y_OFFSET = -68;

local CLASS_SPELLS = {
	["MONK"] = 119582,
	["WARRIOR"] = 2565,
	["PALADIN"] = 53600,
	["HUNTER"] = 190928,
	["ROGUE"] = 185313
};
local SPELL_ID;

local cCharges = CreateFrame("Frame", "cCharges", UIParent);
cCharges:SetPoint("CENTER", UIParent, "CENTER", X_OFFSET, Y_OFFSET);
cCharges:SetHeight(BOX_SIZE);
cCharges:RegisterEvent("ADDON_LOADED");
local _hidden, _at_maxcharges = false, false;

local GetSpellCharges, UnitAffectingCombat = GetSpellCharges, UnitAffectingCombat;
local TEXTURE = [[Interface\ChatFrame\ChatFrameBackground]]
local BACKDROP = {
	bgFile = TEXTURE,
	insets = {top = -1, bottom = -1, left = -1, right = -1}
};

local function OnUpdate(self, elapsed)
  self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed;

  if(self.TimeSinceLastUpdate > 0.2) then
		local a, b, start, duration = GetSpellCharges(SPELL_ID)

		local value = ((GetTime() - start) * 100) / duration;
		self:SetValue(value);

		self.TimeSinceLastUpdate = 0;
  end;
end;

cCharges:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end);

local function fill_bar(bar, value)
	bar:SetScript("OnUpdate", nil);
	bar:SetValue(value);
	bar.TimeSinceLastUpdate = 1;
end;

function cCharges:SPELL_UPDATE_CHARGES()
	local current, max, start, duration = GetSpellCharges(SPELL_ID)
	local i;

	--fill the available charges bars
	if(current > 0) then
		for i = current, 1, -1 do
			fill_bar(bars[i], 100)
		end
	end

	--we're at max, exit, if out of combat also hide the frame
	--note: this would hide the frame if you reach max charges out of combat, but not when in combat (PLAYER_REGEN_ENABLED handles that side)
	if(current == max) then
		_at_maxcharges = true;
		if(not UnitAffectingCombat("player")) then
			_hidden = true;
			cCharges:Hide()
		end;

		return;
	end;

	--we're not at max charges, show the frame
	_at_maxcharges = false;
	if(_hidden) then
		_hidden = false;
		cCharges:Show()
	end;

	--set the next to be filled bar onupdate
	bars[current + 1]:SetScript("OnUpdate", OnUpdate);

	--empty the bars that are past the currenly filling one
	for i = current + 2, max do
		fill_bar(bars[i], 0)
	end
end;

function cCharges:PLAYER_REGEN_DISABLED()
	_hidden = false;
	cCharges:Show()
end;

function cCharges:PLAYER_REGEN_ENABLED()
	--this covers the case were you exit combat at max charges
	if(not _hidden and _at_maxcharges) then
		_hidden = true;
		cCharges:Hide();
	end;
end;

local function create_bar(parent, index)
	local bar = CreateFrame('StatusBar', nil, parent)
	bar:SetSize(BOX_SIZE, BOX_SIZE)
	bar:SetStatusBarTexture(TEXTURE)
	bar:SetBackdrop(BACKDROP)
	bar:SetBackdropColor(0, 0, 0)

	if(index == 1) then
		bar:SetPoint('TOPLEFT', parent, 'BOTTOMLEFT', 0, 0)
	else
		bar:SetPoint('LEFT', parent, 'RIGHT', 4, 0)
	end

	local barBG = bar:CreateTexture(nil, 'BORDER')
	barBG:SetAllPoints()

	bar:SetStatusBarColor(0.52, 1.00, 0.52)
	barBG:SetColorTexture(0.71, 1.00, 0.92, .2)
	bar:SetOrientation("VERTICAL")
	bar:SetMinMaxValues(0, 100);
	bar:SetValue(100);
	bar.TimeSinceLastUpdate = 1;

	return bar
end;

local CREATED = false;
function cCharges:PLAYER_ENTERING_WORLD()
	if(not IsSpellKnown(SPELL_ID) or CREATED) then return end;

	local parent, index = cCharges;
	local current, max, start, duration = GetSpellCharges(SPELL_ID)
	cCharges:SetWidth(max * BOX_SIZE + 4 * (max - 1));

	for index = 1, max do
		bars[index] = create_bar(parent, index)
		parent = bars[index]
	end;

	CREATED = true
	if(not UnitAffectingCombat("player")) then
		_hidden = true;
		cCharges:Hide()
	end;
	--looks like SPELL_UPDATE_COOLDOWN fires at the same time
	cCharges:RegisterEvent("SPELL_UPDATE_CHARGES");
end;

function cCharges:ADDON_LOADED(v)
	local class = select(2, UnitClass("player"));
	if(v ~= "cCharges" or not CLASS_SPELLS[class]) then return end;
	SPELL_ID = CLASS_SPELLS[class];

	--starting the bar creation on PLAYER_ENTERING_WORLD since ADDON_LOADED (when loging-in) fires before spells are ready
	cCharges:RegisterEvent("PLAYER_ENTERING_WORLD");
	cCharges:RegisterEvent("PLAYER_REGEN_DISABLED");
	cCharges:RegisterEvent("PLAYER_REGEN_ENABLED");
end;